/*
   ┏━━━┓╱╱╱╱╱┏┓╱╱┏┓╱╱╱╱╱┏┓╱╱   
   ┃┏━┓┃╱╱╱╱╱┃┃╱╱┃┃╱╱╱╱╱┃┃╱╱╱  
   ┃┗━┛┣┓┏┳━━┫┃┏┓┃┃╱╱┏━━┫┗━┳━━┓
   ┃┏━━┫┃┃┃┏┓┃┗┛┃┃┃╱┏┫┏┓┃┏┓┃━━┫
   ┃┃╱╱┃┗┛┃┃┃┃┏┓┃┃┗━┛┃┏┓┃┗┛┣━━┃
   ┗┛╱╱┗━━┻┛┗┻┛┗┛┗━━━┻┛┗┻━━┻━━┛
    ━━━━━━━━━━━━━━━━━━━━━━━━━━ 

    Copyright (c) 2024 Punk Labs LLC

    This file is part of OneTrick B-BOI

    OneTrick B-BOI is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the Free
    Software Foundation, either version 3 of the License, or (at your option)
    any later version.

    OneTrick B-BOI is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along with
    OneTrick B-BOI.  If not, see <http://www.gnu.org/licenses/>.
*/

declare name "OneTrick B-BOI DSP";
declare copyright "Copyright (c) 2024 Punk Labs LLC";
declare license "GPLv3 (or later)";

import("shared.lib");


micFilter = fi.highpass(2, 500);
adConverter = result with {
    bits = 8;
    samplerate = 9380;
    result = ot.bitcrusher(bits, samplerate);
};
daConverter = result with {
    bits = 9;
    samplerate = 9380;
    result = ot.bitcrusher(bits, samplerate);
};

pitchShifter_Rap1(mode, signal) = result with {
    rapModes = waveform { 2, 6, 8, 10, 14 };
    rapMode = int(mode) : max(0) : min(5);
    actualMode = (rapModes, rapMode) : rdtable;
    result = pitchShifter_OKI_M6322.process(actualMode, signal);
};
// Either something is wrong or this pitch shifter is baddddd...
// Let's stick with the simple version below
pitchShifter_OKI_M6322 = environment {
    semitones = waveform { -12, -9, -8, -7, -5, -4, -3, -1, 0, 1, 3, 4, 5, 7, 8, 9, 12 };
    lpf = waveform { 2.07, 2.53, 2.53, 2.85, 3.26, 3.26, 3.26, 3.8, 3.8, 4.56, 4.56, 5.7, 5.7, 5.7, 5.7, 7.6, 7.6 };
    interval = waveform { 18, 20, 20, 30, 38, 40, 45, 110, 105/*disabled*/, 120, 40, 28, 22, 15, 12, 10, 7 };
    active = waveform { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1 };
    bitsAD = 8;
    bitsDA = 9;
    sampleRate = 9380; // Guess

    process(mode, signal) = result with {
        hwMode = mode : max(0) : min(16);
        transpose = (semitones, hwMode) : rdtable;
        cutoffKhz = (lpf, hwMode) : rdtable;
        bufferMS = (interval, hwMode) : rdtable;
        isActive = (active, hwMode) : rdtable;

        pitchShiftWindow = bufferMS / 1000 * ma.SR;
        pitchShiftCrossFade = 1;
        result =
            signal :
            fi.highpass(2, 500) : // Tiny Microphone
            ot.bitcrusher(bitsAD, sampleRate) :
            ef.transpose(pitchShiftWindow, pitchShiftCrossFade, transpose) :
            fi.lowpass(4, cutoffKhz * 1000) :
            ot.bitcrusher(bitsDA, sampleRate);
    };
};

pitchShifter_Simple(semi, signal) = result with {
    window = 10/1000*ma.SR;
    crossfade=window/4;
    // Save some stack memory since we can go over on Windows:
    lighterTranspose(w, x, s, sig) = de.fdelay(maxDelay,d,sig)*ma.fmin(d/x,1) +
        de.fdelay(maxDelay,d+w,sig)*(1-ma.fmin(d/x,1))
    with {
        maxDelay = 16384-2; // Faust adds 2 for padding... this is 1/4 of the default size
        i = 1 - pow(2, s/12);
        d = i : (+ : +(w) : fmod(_,w)) ~ _;
    };
    result = signal : lighterTranspose(window, crossfade, semi);
};

// // TEST SAMPLE:
// sampleTime = ot.timer.whileHeldTuned(ot.input.gate, ot.input.key);
// fakeMicrophone = ot.sample(0).oneShot.cubic(sampleTime).channel(0) : ot.filterTriggerClicks(ot.input.gate != ot.input.gate');
// microphone = _+fakeMicrophone : micFilter : adConverter;

microphone = micFilter : adConverter;

voice = microphone : pitchShifter_Simple(voiceTranspose + ot.input.pitchShift(12) : ot.smoothParam);

process = voice : processVoice;
